#include <math.h>

#include "mesh.h"
#include "piece.h"

#ifdef DMALLOC
#include<dmalloc.h>
#endif

Piece::Piece() {
	int i;
	for(i=0;i<16;i++)
			transform[i]=(i/4)==(i%4)?1:0;
	action=PA_NONE;
	type=PT_NONE;
	model=0;
}

void Piece::setModel(Entity *entity,enum PieceType type)
{
	model=entity;
	this->type=type;
}

Entity *Piece::getModel()
{
	return model;
}

float *Piece::getTransform()
{
	return transform;
}

void Piece::setPos(float x,float y,float z)
{
	transform[12]=x;
	transform[13]=y;
	transform[14]=z;
	action=PA_NONE;
}

void Piece::setMoveTo(float x,float y,float z,float seconds)
{
	moveTo[0]=x;
	moveTo[1]=y;
	moveTo[2]=z;
	moveToTimeLeft=(int)(seconds*1000);
	action=PA_MOVETO;
	//printf("Setting move to over: %d\n",moveToTimeLeft);
	if(moveToTimeLeft<=0) action=PA_NONE;
}

void Piece::update(long elapsed)
{
	//printf("Updating piece (%d elapsed)\n",elapsed);
	if(action==PA_MOVETO) {
		float ratio=(float)elapsed/(float)moveToTimeLeft;
		//printf("Ratio = %.3f\n",ratio);
		if(ratio>1.0f) ratio=1.0;
		//printf("Ratio = %.3f\n",ratio);
		transform[12]=transform[12]*(1.0f-ratio)+moveTo[0]*ratio;
		transform[13]=transform[13]*(1.0f-ratio)+moveTo[1]*ratio;
		transform[14]=transform[14]*(1.0f-ratio)+moveTo[2]*ratio;
		//printf("Moving piece to %.2f,%.2f,%.2f\n",moveTo[0],moveTo[1],moveTo[2]);
		//printf("Moving piece (%d of %d) @ %.2f,%.2f,%.2f\n",elapsed,moveToTimeLeft,transform[12],transform[13],transform[14]);

		moveToTimeLeft-=elapsed;
		if(moveToTimeLeft<=0) action=PA_NONE;
	}
}
